/**
 * 通話に相当するクラス． インタフェースにするか迷ったけどクラスにしたわ． 通話は二者間に限定し，三者間以上の通話は想定してません． TODO
 * 属性，メソッドやコンストラクタは適宜追加して結構ですが，下記にあるメソッド群はコメントの仕様に従い必ず実装してね．
 * 
 * @author kaiya
 * 
 */
public class Call {
	/**
	 * 一応，電話後もcaller, callee, 料金を保持することを想定して電話中か否かの情報は別途属性として持つ．
	 */
	private boolean isCall=true;
	/**
	 * 電話をかけたひと
	 */
	private Callable caller=null;
	/**
	 * 電話をかけたれた人
	 */
	private Callable callee=null;
	/**
	 * この通話の料金．インスタンス作成時に計算．
	 */
	private int charge=defaultCharge;
	/**
	 * デフォルトの通話料金．
	 */
	private static final int defaultCharge=30;

	/**
	 * 電話通話のインスタンスを作成
	 * @param caller かけた人 null は想定していない．
	 * @param callee かけられた人 null は想定していない．
	 */
	public Call(Callable caller, Callable callee){
		SmartPhone er=null;
		SmartPhone ee=null;
		// ココのキャストがダサい．スマぬ．
		if(caller instanceof SmartPhone && callee instanceof SmartPhone){
			 er=(SmartPhone)caller;
			 ee=(SmartPhone)callee;
		}
		if(er!=null && ee!=null & er.calling()==false && ee.calling()==false){
			this.caller=caller;
			this.callee=callee;
			this.isCall=true;
			charge=defaultCharge;
			if(er.isActiveLine() && ee.isActiveLine()){
				charge=0;
			}
		}else{ // 本来，例外のほうが良いが，通話が成立しない場合．
			this.isCall=false;
			charge=0;
		}
		
	}
	/**
	 * 電話を切る
	 */
	public void hangUp() {
		isCall=false;
		// 切っても caller, callee, chargeの情報は保持している．
	}

	/**
	 * 電話中か否かを判断する．
	 * 
	 * @return 電話中なら真，電話が終了しているなら偽．
	 */
	public boolean isCalling() {
		return isCall;
	}

	/**
	 * 電話料金を返す． 電話中でもリアルタイムに更新されるなら，それはそれで面白いが， 基本，通常電話なら一律30円，
	 * アプリケーションL経由なら一律0円と考えてよい． 電話をする双方がアプリLを有効にしている場合のみ0円となる．
	 * また，電話終了後にも，インスタンスへの参照があるなら，電話料金を参照できるものとする．
	 * 電話代は無論，電話をかけた人が支払うことになり，かけられた人には課金されない．
	 * 
	 * @return 電話料金を返す．
	 */
	public int charge() {
		return charge;
	}

	/**
	 * 電話をかけた人
	 * 
	 * @return 電話をかけた人の電話へのリファレンスを返す
	 */
	public Callable caller() {
		return caller;
	}

	/**
	 * 電話をかけられた人
	 * 
	 * @return 電話をかけられた人の電話へのリファレンスを返す
	 */
	public Callable callee() {
		return callee;
	}
}
