import java.util.ArrayList;

/**
 * スマートフォンに相当するクラス． 
 * 解答例．
 * 
 * @author kaiya
 * 
 */
public class SmartPhone implements Callable, Linable {
	/**
	 * 番号をもとにインスタンスを作成
	 * 
	 * @param n
	 *            電話番号，単純化のために単なる数字，例えば，256 とかを番号とみなす．
	 */
	public SmartPhone(int n) {
		number=n;
	}
	
	/**
	 * 電話番号
	 */
	private int number;
	/**
	 * Lがactivateされてるかどうかを保持．初期状態ではinactive. 
	 */
	private boolean line=false;
	/**
	 * 電話料金の合計量．初期値は当然0円．
	 */
	private int charge=0;
	/**
	 * 電話帳．初期値は空リスト．
	 */
	private ArrayList<Callable> list=new ArrayList<Callable>();
	
	/**
	 * 直近の通話インスタンスへの参照．リストにすれば通話履歴が残せるか．
	 */
	private Call call=null;

	@Override
	public boolean isActiveLine() {
		return line;
	}

	@Override
	public void activateLine() {
		line=true;

	}

	@Override
	public void inactivateLine() {
		line=false;

	}

	@Override
	public Callable[] getFreeAddresses() {
		ArrayList<Callable> fa=new ArrayList<Callable>();
		for(Callable c: list){
			// ★★ ココのinstanceofチェックとキャストは超絶ダサい．
			// インタフェース設計のミスだ．すまぬ．
			if(c instanceof Linable){
				Linable s=(Linable)c;
				if(s.isActiveLine()){
					fa.add(c);
				}
			}
		}
		return fa.toArray(new Callable[fa.size()]);
	}

	@Override
	public Call call(Callable caller) {
		Call c = new Call(caller, this);
		if(c.isCalling()==false){
			return null;
		}
		call=c;
		if(caller instanceof SmartPhone){
			SmartPhone er=(SmartPhone)caller;
			er.addCharge(call.charge());
		}
		return call;
	}
	
	/**
	 * 通話料金を加算する．
	 * コレもcallの引数と適用が逆ならいらないメソッドだな・・・
	 * @param c 加算する料金．
	 */
	public void addCharge(int c){
		charge += c;
	}
	
	/**
	 * 通話中か否かのチェック．本来はCallableに定義すべきだな orz
	 * @return 通話中なら真．
	 */
	public boolean calling(){
		if(call!=null && call.isCalling()) return true;
		return false;
	}

	@Override
	public Callable[] getAddresses() {
		// toArrayの引数の書き方はすぐ忘れてしまう・・・
		return list.toArray(new Callable[list.size()]);
	}

	@Override
	public boolean addAddress(Callable c) {
		if(c!=null && c!=this){
			list.add(c);
			return true;
		}
		return false;
	}

	@Override
	public boolean removeAddress(Callable c) {
		if(c!=null && list.contains(c)){
			list.remove(c);
			return true;
		}
		return false;
	}

	@Override
	public int getTotalCharge() {
		return charge;
	}

	@Override
	public int getNumber() {
		return number;
	}

}
